package xyz.joethei.studium.algodat.praktikum.blatt4;

import java.util.Arrays;

public class Heap {

    private int[] array;
    private int last = 0;
    private int capacity;
    private int swapCount = 0;


    public Heap(int capacity) {
        array = new int[capacity + 1];
        this.capacity = capacity;
    }

    public Heap(int[] array) {
        this.capacity = array.length + 1;
        this.array = array;
        this.last = array.length-1;
        for (int i = indexOf(findLastWithChild()); i >= 0; i--) {
            sinkWithIndex(array[i]);
        }
    }

    private int findLastWithChild() {
        int index = -1;
        for(int i = 0; i < last; i++) {
            if(i * 2 > size()) {
                index = i / 2;
                break;
            }
        }
        return index + 1;
    }

    public int size() {
        return last;
    }

    public boolean isEmpty() {
        return size() == 0;
    }

    public int getSwapCount() {
        return swapCount;
    }

    public int min() throws RuntimeException {
        if (isEmpty())
            throw new RuntimeException("The heap is empty.");
        else
            return  array[1];
    }

    private int compare(int x, int y) {
        return Integer.compare(x, y);
    }

    public void insert(int i) throws RuntimeException {
        if (size() == capacity)
            throw new RuntimeException("Heap overflow.");
        else {
            last++;
            array[last] = i;
            up();
        }
    }

    public int removeMin() throws RuntimeException {
        if (isEmpty())
            throw new RuntimeException("Heap is empty.");
        else {
            int min = min();
            array[1] = array[last];
            last--;
            sinkWithIndex(1);
            return min;
        }
    }

    private void sink(int s) {
        sinkWithIndex(indexOf(s));
    }

    private void sinkWithIndex(int index) {
        while (true) {
            int child = index * 2;
            if (child > size())
                break;
            if (child + 1 <= size()) {
                child = findMin(child, child + 1);
            }
            if (compare(array[index], array[child]) <= 0)
                break;
            swap(index, child);
            index = child;
        }
    }

    private void up() {
        int index = size();
        while (index > 1) {
            int parent = index / 2;
            if (compare(array[index], array[parent]) >= 0)
                break;
            swap(index, parent);
            index = parent;
        }
    }

    private void swap(int i, int j) {
        swapCount++;
        int tmp = array[i];
        array[i] = array[j];
        array[j] = tmp;
    }

    private int findMin(int leftChild, int rightChild) {
        if (compare(array[leftChild], array[rightChild]) <= 0)
            return leftChild;
        else
            return rightChild;
    }

    private int indexOf(int j) {
        int index = -1;
        for (int i = 0; i < array.length; i++) {
            if (array[i] == j) {
                index = i;
            }
        }
        return index;
    }

    @Override
    public String toString() {
        return Arrays.toString(array);
    }
}